clear;
restoredefaultpath();
addpath("../SPIDER_functions/");

%load("Cou-turbprofdata1e8-u0.5.mat");
%load("Cou-turbprofdata1e8-u0.5F.mat");
%L = 1.0;

load("Cou-turbprofdata1e8-u1.0.mat");
%load("Cou-turbprofdata1e8-u1.0F.mat");
L = 0.1;
%L = 0.15;
%L = 0.2;

number_of_library_terms = 3;   %under-estimate this
number_of_windows       = 512; %number of domains we integrate over 
degrees_of_freedom      = 1;   %vectors have one degree of freedom
dimension               = 2;   %how many dimensions does our data have?
envelope_power          = 5;   %weight is (1-x^2)^power
size_vec                = [383,10]; %how many gridpoints should we use per integration?
%size_vec                = [1023,100]; %how many gridpoints should we use per integration?
buffer                  = 0; %Do not use points this close to boundary

U = data(:,:,1);
W = data(:,:,2);
T = data(:,:,3);
R = data(:,:,4);
R11 = data(:,:,5);
R12 = data(:,:,6);
R22 = data(:,:,7);
Fx = data(:,:,8);
Fz = data(:,:,9);
Q = data(:,:,10);
Rpt5 = data(:,:,11);
R1pt5 = data(:,:,12);

dzR11 = diff_dim(R11,z,1);
dzzR11 = diff_dim(dzR11,z,1);
dtR11 = diff_dim(R11,t,2);
dzU = diff_dim(U,z,1);

%define shorthand notation
nl = number_of_library_terms;
nw = number_of_windows;
dof= degrees_of_freedom;

%Make important objects for integration
pol      = envelope_pol( envelope_power, dimension );
G        = zeros( dof*nw, nl );
labels   = cell(nl, 1);
scales   = zeros(1,nl);

%we also need pol to be odd in time. multiply it by t
pol0 = pol; %save for normalization

size_of_data = size(U, 1:dimension);

seed = 1;
corners = pick_subdomains_manual_seed( size_of_data, size_vec, buffer, nw, seed );

grid = {z,t};

%Nondimensionalize and compute mean and variation of data
%TODO;

%%%%%%%%%%%%%%%%%%%%%%
% LIBRARY INTEGRATION
%%%%%%%%%%%%%%%%%%%%%%
a = 1;
invL = 1/L;
invL2 = 1/(L*L);

%labels{a} =  "dR11dt + W.R11_z + 2.R12.U_z - 1e-4.R11_zz";
%G(:,a)    = SPIDER_integrate( dtR11, [], grid, corners, size_vec, pol ) + ...
%            SPIDER_integrate( W.*dzR11, [], grid, corners, size_vec, pol ) + ...
%            2*SPIDER_integrate( R12.*dzU, [], grid, corners, size_vec, pol ) + ...
%            -1e-4*SPIDER_integrate( dzzR11, [], grid, corners, size_vec, pol );
%scales(a) = 1;
%a = a+1;

labels{a} =  "dR11dt + 2.R12.U_z - 1e-4.R11_zz";
G(:,a)    = SPIDER_integrate( dtR11, [], grid, corners, size_vec, pol ) + ...
            2*SPIDER_integrate( R12.*dzU, [], grid, corners, size_vec, pol ) + ...
            -1e-4*SPIDER_integrate( dzzR11, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} =  "Rpt5.R11/L";
G(:,a)    = invL*SPIDER_integrate( Rpt5.*R11, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} =  "-R1pt5/3L";
G(:,a)    = -invL*SPIDER_integrate( R1pt5, [], grid, corners, size_vec, pol )/3.0;
scales(a) = 1;
a = a+1;

labels{a} =  "1e-4*R11/(L*L)";
G(:,a)    = 1e-4*invL2*SPIDER_integrate( R11, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

%normalize with non-odd polynomial.
norm_vec = SPIDER_integrate( 0*U + 1, [], grid, corners, size_vec, pol );      
%norm_vec = repmat( norm_vec, [dof*numel(u0s),1] );

G = G./norm_vec;
G = G./scales;  

function [dF] = diff_dim(F, y, dim)
% 3-point forward/center/backwards differencing to approximate first derivative

    dF = zeros(size(F));

    % boundary points
    y0 = y(1); y1 = y(2); y2 = y(3);
    c0 = (2*y0-y1-y2)/((y0-y1)*(y0-y2));
    c1 = (2*y0-y0-y2)/((y1-y0)*(y1-y2));
    c2 = (2*y0-y0-y1)/((y2-y0)*(y2-y1));
    if dim == 1
        dF(1, :) = c0*F(1, :)+c1*F(2, :)+c2*F(3, :);
    else
        dF(:, 1) = c0*F(:, 1)+c1*F(:, 2)+c2*F(:, 3);
    end

    for i=2:length(y)-1
        y0 = y(i-1); y1 = y(i); y2 = y(i+1);
        c0 = (2*y1-y1-y2)/((y0-y1)*(y0-y2));
        c1 = (2*y1-y0-y2)/((y1-y0)*(y1-y2));
        c2 = (2*y1-y0-y1)/((y2-y0)*(y2-y1));
        if dim == 1
            dF(i, :) = c0*F(i-1, :)+c1*F(i, :)+c2*F(i+1, :);
        else
            dF(:, i) = c0*F(:, i-1)+c1*F(:, i)+c2*F(:, i+1);
        end
    end

    y0 = y(end-2); y1 = y(end-1); y2 = y(end);
    c0 = (2*y2-y1-y2)/((y0-y1)*(y0-y2));
    c1 = (2*y2-y0-y2)/((y1-y0)*(y1-y2));
    c2 = (2*y2-y0-y1)/((y2-y0)*(y2-y1));
    if dim == 1
        dF(end, :) = c0*F(end-2, :)+c1*F(end-1, :)+c2*F(end, :);
    else
        dF(:, end) = c0*F(:, end-2)+c1*F(:, end-1)+c2*F(:, end);
    end

end
