clear;
restoredefaultpath();
addpath("../SPIDER_functions/");

load("3Ddata1e4-long.mat");
U = data(:,:,:,:,1);
V = data(:,:,:,:,2);
W = data(:,:,:,:,3);
P = data(:,:,:,:,4);
T = data(:,:,:,:,5);

dxU = diff_dim(U,x,1);
dyU = diff_dim(U,y,2);
dzU = diff_dim(U,z,3);
dtU = diff_dim(U,t,4);

dxV = diff_dim(V,x,1);
dyV = diff_dim(V,y,2);
dzV = diff_dim(V,z,3);
dtV = diff_dim(V,t,4);

dxW = diff_dim(W,x,1);
dyW = diff_dim(W,y,2);
dzW = diff_dim(W,z,3);
dtW = diff_dim(W,t,4);

dxP = diff_dim(P,x,1);
dyP = diff_dim(P,y,2);
dzP = diff_dim(P,z,3);
dtP = diff_dim(P,t,4);

dxT = diff_dim(T,x,1);
dyT = diff_dim(T,y,2);
dzT = diff_dim(T,z,3);
dtT = diff_dim(T,t,4);

number_of_library_terms = 3;   %under-estimate this
number_of_windows       = 512; %number of domains we integrate over - 3D - START SMALL?
degrees_of_freedom      = 3;   %vectors have one degree of freedom DOES THIS NEED TO BE ALTERED?
dimension               = 4;   %how many dimensions does our data have? TED FOR 3D
envelope_power          = 8;   %weight is (1-x^2)^power
size_vec                = [32,32,64,64]; %how many gridpoints should we use per integration?
buffer                  = 0; %Do not use points this close to boundary

%define shorthand notation
nl = number_of_library_terms;
nw = number_of_windows;
dof= degrees_of_freedom;

%Make important objects for integration
pol      = envelope_pol( envelope_power, dimension );
G        = zeros( dof*nw, nl );
labels   = cell(nl, 1);
scales   = zeros(1,nl);

%we also need pol to be odd in time. multiply it by t
pol0 = pol; %save for normalization

size_of_data = size(U, 1:dimension);

seed = 1;
corners = pick_subdomains_manual_seed( size_of_data, size_vec, buffer, nw, seed );

grid = {x,y,z,t};

%Nondimensionalize and compute mean and variation of data
%TODO;

%%%%%%%%%%%%%%%%%%%%%%
% LIBRARY INTEGRATION
%%%%%%%%%%%%%%%%%%%%%%
a = 1;

labels{a} =  "U";
G(:,a)    = SPIDER_integrate_vector( U,V,W, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} =  "U_t";
G(:,a)    = SPIDER_integrate_vector( U,V,W, [4], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "\nabla_i P";
G(:,a)    =[SPIDER_integrate( P, [1], grid, corners, size_vec, pol );
            SPIDER_integrate( P, [2], grid, corners, size_vec, pol );
            SPIDER_integrate( P, [3], grid, corners, size_vec, pol );];
scales(a) = 1;
a = a+1;

labels{a} = "\nabla_i T";
G(:,a)    =[SPIDER_integrate( T, [1], grid, corners, size_vec, pol );
            SPIDER_integrate( T, [2], grid, corners, size_vec, pol );
            SPIDER_integrate( T, [3], grid, corners, size_vec, pol );];
scales(a) = 1;
a = a+1;

labels{a} =  "P.*U";
G(:,a)    = SPIDER_integrate_vector( P.*U, P.*V, P.*W, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} =  "T.*U";
G(:,a)    = SPIDER_integrate_vector( T.*U, T.*V, T.*W, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "(U_j.\nabla_j) U_i"; % = \nabla_j (U_j U_i) via conservative form
G(:,a)    = SPIDER_integrate_vector( U.*U, U.*V, U.*W, [1], grid, corners, size_vec, pol ) + ...
            SPIDER_integrate_vector( V.*U, V.*V, V.*W, [2], grid, corners, size_vec, pol ) + ...
            SPIDER_integrate_vector( W.*U, W.*V, W.*W, [3], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} =  "\nabla^2 u_i";
G(:,a)    = SPIDER_integrate_vector( U, V, W, [1,1], grid, corners, size_vec, pol ) + ...
            SPIDER_integrate_vector( U, V, W, [2,2], grid, corners, size_vec, pol ) + ...
            SPIDER_integrate_vector( U, V, W, [3,3], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} =  "U_tt";
G(:,a)    = SPIDER_integrate_vector( U,V,W, [4,4], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "U^2.U";
U_sq = U.^2 + V.^2 + W.^2;
G(:,a)    = SPIDER_integrate_vector( U_sq.*U, U_sq.*V, U_sq.*W, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "P^2.U";
G(:,a)    = SPIDER_integrate_vector( P.*P.*U, P.*P.*V, P.*P.*W, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "T^2.U";
G(:,a)    = SPIDER_integrate_vector( T.*T.*U, T.*T.*V, T.*T.*W, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "(\nabla_i P)_t";
G(:,a)    = SPIDER_integrate_vector( dxP, dyP, dzP, [4], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "(\nabla_i T)_t";
G(:,a)    = SPIDER_integrate_vector( dxT, dyT, dzT, [4], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "P \nabla_i P";
G(:,a)    = SPIDER_integrate_vector( P.*dxP, P.*dyP, P.*dzP, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "T \nabla_i T";
G(:,a)    = SPIDER_integrate_vector( T.*dxT, T.*dyT, T.*dzT, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "U.(\nabla.U)";
grad_U = dxU + dyU + dzU;
G(:,a)    = SPIDER_integrate_vector( grad_U.*U, grad_U.*V, grad_U.*W, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "\nabla(\nabla.U)";
G(:,a)    =[SPIDER_integrate( grad_U, [1], grid, corners, size_vec, pol );
            SPIDER_integrate( grad_U, [2], grid, corners, size_vec, pol );
            SPIDER_integrate( grad_U, [3], grid, corners, size_vec, pol );];
scales(a) = 1;
a = a+1;

labels{a} =  "P U_t";
G(:,a)    = SPIDER_integrate_vector( P.*dtU, P.*dtV, P.*dtW, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} =  "T U_t";
G(:,a)    = SPIDER_integrate_vector( T.*dtU, T.*dtV, T.*dtW, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "U P_t";
G(:,a)    = SPIDER_integrate_vector( U.*dtP, V.*dtP, W.*dtP, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "U T_t";
G(:,a)    = SPIDER_integrate_vector( U.*dtT, V.*dtT, W.*dtT, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} = "P e_z";
G(:,a)    =[0*SPIDER_integrate( P, [], grid, corners, size_vec, pol );
            0*SPIDER_integrate( P, [], grid, corners, size_vec, pol );
            SPIDER_integrate( P, [], grid, corners, size_vec, pol );];
scales(a) = 1;
a = a+1;

labels{a} = "T e_z";
G(:,a)    =[0*SPIDER_integrate( T, [], grid, corners, size_vec, pol );
            0*SPIDER_integrate( T, [], grid, corners, size_vec, pol );
            SPIDER_integrate( T, [], grid, corners, size_vec, pol );];
scales(a) = 1;
a = a+1;

labels{a} = "\nabla_i U^2";
G(:,a)    =[SPIDER_integrate( U_sq, [1], grid, corners, size_vec, pol );
            SPIDER_integrate( U_sq, [2], grid, corners, size_vec, pol );
            SPIDER_integrate( U_sq, [3], grid, corners, size_vec, pol );];
scales(a) = 1;
a = a+1;

labels{a} = "\nabla_i P^2";
G(:,a)    =[SPIDER_integrate( P.*P, [1], grid, corners, size_vec, pol );
            SPIDER_integrate( P.*P, [2], grid, corners, size_vec, pol );
            SPIDER_integrate( P.*P, [3], grid, corners, size_vec, pol );];
scales(a) = 1;
a = a+1;

labels{a} = "\nabla_i T^2";
G(:,a)    =[SPIDER_integrate( T.*T, [1], grid, corners, size_vec, pol );
            SPIDER_integrate( T.*T, [2], grid, corners, size_vec, pol );
            SPIDER_integrate( T.*T, [3], grid, corners, size_vec, pol );];
scales(a) = 1;
a = a+1;

%% normalize with non-odd polynomial.
norm_vec = SPIDER_integrate( 0*U + 1, [], grid, corners, size_vec, pol );      
norm_vec = repmat( norm_vec, [dof,1] );

G = G./norm_vec;
G = G./scales;

function d=SPIDER_integrate_vector( U,V,W, deriv, grid, corners, size_vec, pol )
   a = SPIDER_integrate( U, deriv, grid, corners, size_vec, pol );
   b = SPIDER_integrate( V, deriv, grid, corners, size_vec, pol ); 
   c = SPIDER_integrate( W, deriv, grid, corners, size_vec, pol );
   d = [a;b;c];
end

function [dF] = diff_dim(F, y, dim)
% 3-point forward/center/backwards differencing to approximate first derivative

    dF = zeros(size(F));

    % boundary points
    y0 = y(1); y1 = y(2); y2 = y(3);
    c0 = (2*y0-y1-y2)/((y0-y1)*(y0-y2));
    c1 = (2*y0-y0-y2)/((y1-y0)*(y1-y2));
    c2 = (2*y0-y0-y1)/((y2-y0)*(y2-y1));
    if dim == 1
      dF(1, :, :, :) = c0*F(1, :, :, :)+c1*F(2, :, :, :)+c2*F(3, :, :, :);
    elseif dim == 2
      dF(:, 1, :, :) = c0*F(:, 1, :, :)+c1*F(:, 2, :, :)+c2*F(:, 3, :, :);
    elseif dim == 3
      dF(:, :, 1, :) = c0*F(:, :, 1, :)+c1*F(:, :, 2, :)+c2*F(:, :, 3, :);
    else
      dF(:, :, :, 1) = c0*F(:, :, :, 1)+c1*F(:, :, :, 2)+c2*F(:, :, :, 3);
    end

    for i=2:length(y)-1
        y0 = y(i-1); y1 = y(i); y2 = y(i+1);
        c0 = (2*y1-y1-y2)/((y0-y1)*(y0-y2));
        c1 = (2*y1-y0-y2)/((y1-y0)*(y1-y2));
        c2 = (2*y1-y0-y1)/((y2-y0)*(y2-y1));
        if dim == 1
	  dF(i, :, :, :) = c0*F(i-1, :, :, :)+c1*F(i, :, :, :)+c2*F(i+1, :, :, :);
        elseif dim == 2
	  dF(:, i, :, :) = c0*F(:, i-1, :, :)+c1*F(:, i, :, :)+c2*F(:, i+1, :, :);
        elseif dim == 3
	  dF(:, :, i, :) = c0*F(:, :, i-1, :)+c1*F(:, :, i, :)+c2*F(:, :, i+1, :);
        else
	  dF(:, :, :, i) = c0*F(:, :, :, i-1)+c1*F(:, :, :, i)+c2*F(:, :, :, i+1);
        end
    end

    y0 = y(end-2); y1 = y(end-1); y2 = y(end);
    c0 = (2*y2-y1-y2)/((y0-y1)*(y0-y2));
    c1 = (2*y2-y0-y2)/((y1-y0)*(y1-y2));
    c2 = (2*y2-y0-y1)/((y2-y0)*(y2-y1));
    if dim == 1
      dF(end, :, :, :) = c0*F(end-2, :, :, :)+c1*F(end-1, :, :, :)+c2*F(end, :, :, :);
    elseif dim == 2
      dF(:, end, :, :) = c0*F(:, end-2, :, :)+c1*F(:, end-1, :, :)+c2*F(:, end, :, :);
    elseif dim == 3
      dF(:, :, end, :) = c0*F(:, :, end-2, :)+c1*F(:, :, end-1, :)+c2*F(:, :, end, :);
    else
      dF(:, :, :, end) = c0*F(:, :, :, end-2)+c1*F(:, :, :, end-1)+c2*F(:, :, :, end);
    end

end
