clear;
restoredefaultpath();
addpath("../SPIDER_functions/");

load("3dturbprofiledata1e4.mat");
%L = 0.08; % eddy length scale estimated from data
%L = 0.15;
L = 0.2;

number_of_library_terms = 3;   %under-estimate this
number_of_windows       = 128; %number of domains we integrate over 
degrees_of_freedom      = 1;   %vectors have one degree of freedom
dimension               = 2;   %how many dimensions does our data have?
envelope_power          = 10;   %weight is (1-x^2)^power
size_vec                = [171,10]; %how many gridpoints should we use per integration?
buffer                  = 10; %Do not use points this close to boundary

U = data(:,:,1);
V = data(:,:,2);
W = data(:,:,3);
T = data(:,:,4);
R = data(:,:,5);
R11 = data(:,:,6);
R12 = data(:,:,7);
R13 = data(:,:,8);
R22 = data(:,:,9);
R23 = data(:,:,10);
R33 = data(:,:,11);
Fx = data(:,:,12);
Fy = data(:,:,13);
Fz = data(:,:,14);
Q = data(:,:,15);
Rpt5 = data(:,:,16);
R1pt5 = data(:,:,17);

dzQ = diff_dim(Q,z,1);
dzzQ = diff_dim(dzQ,z,1);
dtQ = diff_dim(Q,t,2);
dzT = diff_dim(T,z,1);

%define shorthand notation
nl = number_of_library_terms;
nw = number_of_windows;
dof= degrees_of_freedom;

%Make important objects for integration
pol      = envelope_pol( envelope_power, dimension );
G        = zeros( dof*nw, nl );
labels   = cell(nl, 1);
scales   = zeros(1,nl);

%we also need pol to be odd in time. multiply it by t
pol0 = pol; %save for normalization

size_of_data = size(U, 1:dimension);

seed = 1;
corners = pick_subdomains_manual_seed( size_of_data, size_vec, buffer, nw, seed );

grid = {z,t};

%Nondimensionalize and compute mean and variation of data
%TODO;

%%%%%%%%%%%%%%%%%%%%%%
% LIBRARY INTEGRATION
%%%%%%%%%%%%%%%%%%%%%%
a = 1;
invL = 1/L;
invL2 = 1/(L*L);

labels{a} =  "dQdt + W.Q_z + 2.Fz.T_z - 1e-3.Q_zz";
G(:,a)    = SPIDER_integrate( dtQ, [], grid, corners, size_vec, pol ) + ...
            SPIDER_integrate( W.*dzQ, [], grid, corners, size_vec, pol ) + ...
            2*SPIDER_integrate( Fz.*dzT, [], grid, corners, size_vec, pol ) + ...
            -1e-3*SPIDER_integrate( dzzQ, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

%labels{a} =  "dQdt + 2.Fz.T_z - 1e-3.Q_zz";
%G(:,a)    = SPIDER_integrate( dtQ, [], grid, corners, size_vec, pol ) + ...
%            2*SPIDER_integrate( Fz.*dzT, [], grid, corners, size_vec, pol ) + ...
%            -1e-3*SPIDER_integrate( dzzQ, [], grid, corners, size_vec, pol );
%scales(a) = 1;
%a = a+1;

labels{a} =  "Rpt5.Q/L";
G(:,a)    = invL*SPIDER_integrate( Rpt5.*Q, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

labels{a} =  "kappa*Q/L^2";
G(:,a)    = 1e-3 * invL2 * SPIDER_integrate( Q, [], grid, corners, size_vec, pol );
scales(a) = 1;
a = a+1;

%normalize with non-odd polynomial.
norm_vec = SPIDER_integrate( 0*U + 1, [], grid, corners, size_vec, pol );      
%norm_vec = repmat( norm_vec, [dof*numel(u0s),1] );

G = G./norm_vec;
G = G./scales;  

function [dF] = diff_dim(F, y, dim)
% 3-point forward/center/backwards differencing to approximate first derivative

    dF = zeros(size(F));

    % boundary points
    y0 = y(1); y1 = y(2); y2 = y(3);
    c0 = (2*y0-y1-y2)/((y0-y1)*(y0-y2));
    c1 = (2*y0-y0-y2)/((y1-y0)*(y1-y2));
    c2 = (2*y0-y0-y1)/((y2-y0)*(y2-y1));
    if dim == 1
        dF(1, :) = c0*F(1, :)+c1*F(2, :)+c2*F(3, :);
    else
        dF(:, 1) = c0*F(:, 1)+c1*F(:, 2)+c2*F(:, 3);
    end

    for i=2:length(y)-1
        y0 = y(i-1); y1 = y(i); y2 = y(i+1);
        c0 = (2*y1-y1-y2)/((y0-y1)*(y0-y2));
        c1 = (2*y1-y0-y2)/((y1-y0)*(y1-y2));
        c2 = (2*y1-y0-y1)/((y2-y0)*(y2-y1));
        if dim == 1
            dF(i, :) = c0*F(i-1, :)+c1*F(i, :)+c2*F(i+1, :);
        else
            dF(:, i) = c0*F(:, i-1)+c1*F(:, i)+c2*F(:, i+1);
        end
    end

    y0 = y(end-2); y1 = y(end-1); y2 = y(end);
    c0 = (2*y2-y1-y2)/((y0-y1)*(y0-y2));
    c1 = (2*y2-y0-y2)/((y1-y0)*(y1-y2));
    c2 = (2*y2-y0-y1)/((y2-y0)*(y2-y1));
    if dim == 1
        dF(end, :) = c0*F(end-2, :)+c1*F(end-1, :)+c2*F(end, :);
    else
        dF(:, end) = c0*F(:, end-2)+c1*F(:, end-1)+c2*F(:, end);
    end

end
