import h5py
import numpy as np
import matplotlib
#matplotlib.use('Agg')
import matplotlib.pyplot as plt
#from dedalus.extras import plot_tools
from mpl_toolkits.mplot3d import Axes3D
#from sklearn.linear_model import Lasso
from scipy.io import loadmat, savemat
#from sklearn.metrics import mean_squared_error
from scipy.integrate import solve_ivp
from array import array
#from STLSQ_functions import *
from scipy.interpolate import interp2d 

#import pysindy as ps

# Ignore matplotlib deprecation warnings
import warnings
warnings.filterwarnings("ignore", category=UserWarning)

file_range = [1,2,3,4,5,6,7,8]

time = np.array([]) # read in the simulation time
nx, nz = 1024, 384
nt = int(50*file_range[-1])
#nt = int(50*len(file_range))

Pres_store = np.zeros((nt, nx, nz), dtype=np.float64)
Temp_store = np.zeros((nt, nx, nz), dtype=np.float64)
U_store = np.zeros((nt, nx, nz), dtype=np.float64)
W_store = np.zeros((nt, nx, nz), dtype=np.float64)

for qq, ii in enumerate(file_range):
    with h5py.File('snapshots_s{}.h5'.format(ii)) as file:
        print("Doing file", ii)
        if ii==1:
            print(file["scales"].keys())
            xdomain = np.array(file["tasks/pressure"].dims[1][0])
            zdomain = np.array(file["tasks/pressure"].dims[2][0])
        # Time stamps
        time = np.append(time, file["scales/sim_time"][:])
        
        print(len(time))
        # Full data
        if qq+50<nt:
            Pres_store[qq*50:(qq+1)*50,:,:] = file['tasks/pressure'][:,:,:]  # Pressure
            Temp_store[qq*50:(qq+1)*50,:,:] = file['tasks/buoyancy'][:,:,:]  # Temperature / buoyancy
            U_store[qq*50:(qq+1)*50,:,:] = file['tasks/u_x'][:,:,:]          # u_x; x-velocity
            W_store[qq*50:(qq+1)*50,:,:] = file['tasks/u_z'][:,:,:]         # u_z; z-velocity
        else:
            Pres_store[qq*50:,:,:] = file['tasks/pressure'][:,:,:]  # Pressure
            Temp_store[qq*50:,:,:] = file['tasks/buoyancy'][:,:,:]  # Temperature / buoyancy
            U_store[qq*50:,:,:] = file['tasks/u_x'][:,:,:]          # u_x; x-velocity
            W_store[qq*50:,:,:] = file['tasks/u_z'][:,:,:]         # u_z; z-velocity

# Transpose axes into what SINDy expects (time,x,z) -> (x,z,time)
Pres = np.transpose(Pres_store, (1,2,0))
Temp = np.transpose(Temp_store, (1,2,0))
U = np.transpose(U_store, (1,2,0))
W = np.transpose(W_store, (1,2,0))

del Pres_store, Temp_store, U_store, W_store

print('Temp has shape ',Temp.shape,' and size ',Temp.size)

'''
# interpolation from Chebyshev to Fourier
print(len(time))
new_z = np.linspace(0,1,385)[:-1] # Uniform z space
new_Pres = np.zeros((nx, nz, nt), dtype=np.float64)
new_Temp = np.zeros((nx, nz, nt), dtype=np.float64)
new_U = np.zeros((nx, nz, nt), dtype=np.float64)
new_W = np.zeros((nx, nz, nt), dtype=np.float64)
for ii in range(len(time)):
    # interpolate
    interp_p = interp2d(np.squeeze(xdomain),np.squeeze(zdomain),Pres[:,:,ii].T)
    new_Pres[:,:,ii] = interp_p(np.squeeze(xdomain),new_z).T
    interp_t = interp2d(np.squeeze(xdomain),np.squeeze(zdomain),Temp[:,:,ii].T)
    new_Temp[:,:,ii] = interp_t(np.squeeze(xdomain),new_z).T
    interp_U = interp2d(np.squeeze(xdomain),np.squeeze(zdomain),U[:,:,ii].T)
    new_U[:,:,ii] = interp_U(np.squeeze(xdomain),new_z).T
    interp_W = interp2d(np.squeeze(xdomain),np.squeeze(zdomain),W[:,:,ii].T)
    new_W[:,:,ii] = interp_W(np.squeeze(xdomain),new_z).T
'''

#print(new_Pres.shape)
#print(new_Temp.shape)
#print(new_U.shape)
#print(new_W.shape)

# Test - interpolation results seems OK!
#print('Old: ',xdomain[500],zdomain[190],Pres[500,190,5])
#print('Old: ',xdomain[500],zdomain[191],Pres[500,191,5])
#print('New: ',xdomain[500],new_z[190],new_Pres[500,190,5])
#exit()

# Use interpolated data
#Pres = new_Pres
#Temp = new_Temp
#U = new_U
#W = new_W

# Use new uniform z domain
#zdomain = new_z

# SINDy style full data in single array
u_shaped_noiseless = np.zeros((U.shape[0], U.shape[1], U.shape[2], 4))
u_shaped_noiseless[:,:,:,0] = U
u_shaped_noiseless[:,:,:,1] = W
u_shaped_noiseless[:,:,:,2] = Pres
u_shaped_noiseless[:,:,:,3] = Temp

print(u_shaped_noiseless.shape)


########################
# MATT'S MODIFICATIONS
########################

#subsample here directly
u_shaped_noiseless = u_shaped_noiseless[512:, :,200:,:]
xdomain = xdomain[512:]
time = time[200:]

save_dict = { "data": u_shaped_noiseless, "x": xdomain, "z": zdomain, "t": time }
savemat( "data1e10.mat", save_dict )